/**************************************************************************
*
* Filename:    streamTests.cpp
*
* Copyright:   Pico Technology Limited 2006
*
* Author:      MAS
*
* Description:
*   This is a GUI application that demonstrates how to use the
*   PS3000 driver for fast streaming.
*
* Example:
*   Fast streaming with or without triggering 
*   Enable / disable the auto stop functionality
*   Space bar to start / stop the sampling
*   'o' to open the unit
*   Left mouse button to draw a rectangle for zoom in on data
*   Reset data to the original appearance after zooming in
*   Save unaggregated data (not fully implemented)
*   
* History:
*     04May06 MAS Created
*
* Revision Info: "file %n date %f revision %v"
*						""
*
***************************************************************************/

// Windows Header Files:
#include <windows.h>
#include <windowsx.h>
// C RunTime Header Files
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <tchar.h>
#include <stdio.h>
#include <math.h>

#include "streamingTestResource.h"
#include "ps3000.h"

#define TIMER_ID 100
#define TIMER_INTERUPT 100

#define MAX_LOADSTRING 1000

#define TIME_INTERVAL 1
#define TIME_UNIT PS3000_MS
#define aggregation 100
#define MAX_CHANNEL_SAMPLES 100000

#define MAX_SAMPLES 1000
#define MAX_READ_SAMPLES 1000  
#define HEIGHT 200
#define WIDTH 267
#define MAX_COUNT_CHARS 11
#define MAX_STRING_CHARS 240
#define FROM_RIGHT 20
#define FROM_TOP 10
#define FROM_BOTTOM 10
#define FROM_LEFT 20
#define SAMPLE_TAKEN_MSG "Samples Taken:"

#define STREAMING_CFG "Streaming.cfg"
#define MAX_CHARS 80
static char *szSection[] = {"TriggerSettings", "ChannelSettings", "SampleSettings"};
static char *szTrigger[] = {"Enabled", "Channel", "Direction", "Threshold", "Delay", "AutoTrigger", "AutoTriggerTime"};
static char *szChannel[] = {"Enabled%d", "Range%d", "Coupling%d"};
static char *szSample[]  = {"Interval", "Units", "NoOfSample", "Numerator", "Denominator", "AutoStop"};

static short g_trigger;
static unsigned long g_triggerIndex;
static short g_overflow;
static unsigned long g_recieved;
static bool g_bRecieved;

typedef struct tTriggerSettings
{
	short enabled;
	PS3000_CHANNEL channel;
	PS3000_TDIR direction;
	short threshold;
	short delay;
	short auto_trigger;
	long auto_trigger_time;
} TRIGGER_SETTINGS;

typedef struct tChannelSetting
{
	short enabled;
	PS3000_RANGE range;
	short coupling;
} CHANNEL_SETTING;

typedef struct tSampleSettings
{
	unsigned long interval;
	PS3000_TIME_UNITS time_unit;
	unsigned long no_of_samples;
	unsigned long numerator;
	unsigned long denominator;
	short auto_stop;
} SAMPLE_SETTINGS;

typedef struct tDeviceSettings
{
	SAMPLE_SETTINGS sample_setting;
	CHANNEL_SETTING channel_Setting[PS3000_MAX_CHANNELS];
	TRIGGER_SETTINGS trigger_setting;
} DEVICE_SETTINGS;

typedef struct tWindowDisplay
{
	short buffer_max_a[MAX_SAMPLES];
	short buffer_min_a[MAX_SAMPLES];
	short buffer_max_b[MAX_SAMPLES];
	short buffer_min_b[MAX_SAMPLES];
	short buffer_max_c[MAX_SAMPLES];
	short buffer_min_c[MAX_SAMPLES];
	short buffer_max_d[MAX_SAMPLES];
	short buffer_min_d[MAX_SAMPLES];
} WINDOW_DISPLAY;

static WINDOW_DISPLAY g_display;
static DEVICE_SETTINGS g_DeviceSetting;


#define MIN(a,b) (((a) < (b)) ? (a) : (b))

unsigned long g_NumberOfSamples;
unsigned long g_GrandNumberOfSamples;
short         g_Handle;
double numerator, denominator;
double g_TimeFromStart;

static INT_PTR CALLBACK TriggerDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
static INT_PTR CALLBACK ChannelDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
static INT_PTR CALLBACK SamplingDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
void load_config_settings(void);
void save_config_settings(void);

void StreamingGetOverviewBuffersMaxMin (
										short **overviewBuffers,
										short overflow,
										unsigned long triggeredAt,
										short triggered,
										short auto_stop,
										unsigned long nValues);


// Global Variables:
HINSTANCE hInst;								// current instance
TCHAR szTitle[MAX_LOADSTRING];					// The title bar text
TCHAR szWindowClass[MAX_LOADSTRING];			// the main window class name

// Forward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About(HWND, UINT, WPARAM, LPARAM);
HWND g_hWnd;
BOOL g_StopFlag;

static short volt_to_adc(short voltage, short range)
{
	float voltageRange[] = {10,20,50,100,200,500,1000,2000,5000,10000,20000};

	if(voltage > voltageRange[range])
		return PS3000_MAX_VALUE;

	if(voltage < -voltageRange[range])
		return PS3000_MIN_VALUE;

	return (short)((float)voltage * (float)PS3000_MAX_VALUE / voltageRange[range]);
}

static void draw_count_text(HDC hdc, int x, int y)
{
	char stringMsg[MAX_STRING_CHARS + MAX_COUNT_CHARS];
	char stringCount[MAX_COUNT_CHARS];
	int strLth, strCount;
	unsigned long i;

	SetTextAlign(hdc, TA_RIGHT | TA_TOP);
	ZeroMemory(stringMsg, MAX_STRING_CHARS + MAX_COUNT_CHARS);
	ZeroMemory(stringCount, MAX_COUNT_CHARS);
	strLth = sizeof(SAMPLE_TAKEN_MSG) - 1;
	sprintf_s(stringMsg, sizeof(stringMsg) / sizeof(stringMsg[0]), "%s", SAMPLE_TAKEN_MSG);
	sprintf_s(stringCount, sizeof(stringCount) / sizeof(stringCount[0]), "%u", g_GrandNumberOfSamples);
	strCount = (int)strlen(stringCount);
	strCount = (MAX_COUNT_CHARS - 1) - strCount;
	sprintf_s(stringCount, " ");
	for(i = strCount; i > 0; i--)
	{
		strcat_s(stringCount, sizeof(stringCount) / sizeof(stringCount[0]), "0");
	}
	strLth += MAX_COUNT_CHARS;
	sprintf_s(stringMsg, sizeof(stringMsg) / sizeof(stringMsg[0]), "%s%s%u", SAMPLE_TAKEN_MSG, stringCount, g_GrandNumberOfSamples);
	TextOut(hdc, x , y, stringMsg, strLth);

	if(g_overflow)
	{
		char stringMsgOv[]= {"Ov"};

		TextOut(hdc, 30, y, stringMsgOv, 2);
	}

	if(g_bRecieved)
	{
		char stringMsg[]= {"Unagregated samples recieved: %u"};
		char str[80];
		sprintf_s(str, sizeof(str) / sizeof(str[0]) ,stringMsg, g_recieved);

		TextOut(hdc, 450, y, str, (int)strlen(str));
	}
}

static void draw_grid(HDC hdc, RECT rect)
{
	HPEN hpen, oldPen;

	hpen = CreatePen ( PS_DOT, 0, 0x000000 );
	oldPen = (HPEN) SelectObject ( hdc, hpen );
	MoveToEx (hdc, rect.left, rect.bottom / 2, (LPPOINT) NULL);
	LineTo (hdc, rect.right, rect.bottom / 2 );

	for(int i=0 ; i < 9 ; i++)
	{
		MoveToEx ( hdc, (rect.right / 10) * (i+1), rect.top, (LPPOINT) NULL );
		LineTo ( hdc, (rect.right / 10) * (i+1),  rect.bottom);
	}

	DeleteObject ( SelectObject ( hdc, oldPen ) );
}

static void draw_trigger_point(HDC hdc, RECT rect, long yPos)
{
#define TRIGGER_WIDTH 6
	COLORREF colRef = 0x00AABBCC;
	HPEN hpen, oldPen;
	int x1, x2, x3, y1, y2, y3;
	short adcThreshold;
	POINT points[4];

	if(!g_DeviceSetting.trigger_setting.enabled && yPos + 1 >= 0)
		return;

	adcThreshold = volt_to_adc(g_DeviceSetting.trigger_setting.threshold, 
		g_DeviceSetting.channel_Setting[g_DeviceSetting.trigger_setting.channel].range); 

	// find the exact position
	x1 = (int)(((double)(rect.right - rect.left) / (double)MAX_SAMPLES)  * (double)(MAX_SAMPLES - (g_NumberOfSamples - (yPos + 1))));	

	//make the trigger point larger with x1 at the centre
	x2 = x1;
	x3 = x1 + TRIGGER_WIDTH / 2;
	x1 -= TRIGGER_WIDTH / 2;

	y1 = (int)((double)((rect.bottom - rect.top) / 2) - (double)(rect.bottom - rect.top) / (double)(32767 * 2) * adcThreshold);
	//make the trigger point larger with y1 at the centre
	y2 = y1 - TRIGGER_WIDTH / 2;
	y3 = y1 + TRIGGER_WIDTH / 2;

	// create points for polygon
	points[0].x = x1;
	points[0].y = y1;

	points[1].x = x2;
	points[1].y = y2;

	points[2].x = x3;
	points[2].y = y1;

	points[3].x = x2;
	points[3].y = y3;

	hpen = CreatePen ( PS_SOLID, 0, colRef);
	oldPen = (HPEN)SelectObject (hdc, hpen);

	Polygon(hdc, points, 4);

	DeleteObject(SelectObject(hdc, oldPen));
}

static void draw_data_line(HDC hdc, short* buffer_max, short* buffer_min, unsigned long copiedValues, RECT rect, COLORREF colRef)
{
	HPEN hpen, oldPen;
	unsigned long j;
	int x1, x2, y1, y2;

	hpen = CreatePen ( PS_SOLID, 0, colRef);
	oldPen = (HPEN)SelectObject (hdc, hpen);

	x1 = (int)(((double)(rect.right - rect.left) / (double)MAX_SAMPLES)  * (double)(MAX_SAMPLES - g_NumberOfSamples));

	if(buffer_min[0] == PS3000_LOST_DATA)
		y1 = PS3000_LOST_DATA;
	else
		y1 = (int)((double)((rect.bottom - rect.top) / 2) - (double)(rect.bottom - rect.top) / (double)(32767 * 2) * buffer_min[0]);

	if(buffer_min[0] == PS3000_LOST_DATA)
		y2 = PS3000_LOST_DATA;
	else
		y2 = (int)((double)((rect.bottom - rect.top) / 2) - (double)(rect.bottom - rect.top) / (double)(32767 * 2) * buffer_max[0]);

	if(y1 != PS3000_LOST_DATA && y2 != PS3000_LOST_DATA)
	{
		MoveToEx ( hdc, x1, y1, (LPPOINT) NULL );
		LineTo ( hdc, x1, y2);
	}


	for (j = 1; j < copiedValues, j < g_NumberOfSamples; j++)
	{
		//test[j] = buffer_min[j];
		x2 = (int)(((double)(rect.right - rect.left) / (double)MAX_SAMPLES)  * (double)(MAX_SAMPLES - (g_NumberOfSamples - j)));	
		//x2 = (int)(((double)(rect.right - rect.left) / (double)copiedValues)* (double)j);
		if(buffer_min[j] == PS3000_LOST_DATA)
			y1 = PS3000_LOST_DATA;
		else
			y1 = (int)((double)((rect.bottom - rect.top) / 2) - (double)(rect.bottom - rect.top) / (double)(32767 * 2) * buffer_min[j]);


		if(y1 != PS3000_LOST_DATA && y2 != PS3000_LOST_DATA)
		{
			MoveToEx ( hdc, x1, y2, (LPPOINT) NULL );
			LineTo ( hdc, x2, y1);
		}

		if(buffer_max[j] == PS3000_LOST_DATA)
			y2 = PS3000_LOST_DATA;
		else
			y2 = (int)((double)((rect.bottom - rect.top) / 2) - (double)(rect.bottom - rect.top) / (double)(32767 * 2) * buffer_max[j]);

		if(y1 != PS3000_LOST_DATA && y2 != PS3000_LOST_DATA)
		{
			MoveToEx ( hdc, x2, y1, (LPPOINT) NULL );
			LineTo ( hdc, x2, y2);
		}
		x1 = x2;
		//		y1 = y2;
	}
	DeleteObject(SelectObject(hdc, oldPen));
}


static long reset_data(WINDOW_DISPLAY * display, unsigned long *triggerAt, short *triggered)
{
	unsigned long copiedValues;

	short pbuffer_a_min[MAX_SAMPLES];
	short pbuffer_a_max[MAX_SAMPLES];
	short pbuffer_b_min[MAX_SAMPLES];
	short pbuffer_b_max[MAX_SAMPLES];
	short pbuffer_c_min[MAX_SAMPLES];
	short pbuffer_c_max[MAX_SAMPLES];
	short pbuffer_d_min[MAX_SAMPLES];
	short pbuffer_d_max[MAX_SAMPLES];
	unsigned long no_of_values = MAX_SAMPLES;
	double	start_time = 0;
	short overflow = 0;
	double time_interval_ns;

	switch(g_DeviceSetting.sample_setting.time_unit)
	{
	case PS3000_NS:
		time_interval_ns = g_DeviceSetting.sample_setting.interval;
		break;
	case PS3000_US:
		time_interval_ns = g_DeviceSetting.sample_setting.interval * 1000;
		break;
	case PS3000_MS:
		time_interval_ns = g_DeviceSetting.sample_setting.interval * 1000 * 1000;
		break;
	default:
		time_interval_ns = g_DeviceSetting.sample_setting.interval;
	}

	denominator = g_DeviceSetting.sample_setting.denominator;

	double ClientAreaStartTime = 0;
	if(g_DeviceSetting.trigger_setting.enabled)
		ClientAreaStartTime =  denominator * time_interval_ns * ((double)MAX_SAMPLES * ((double)g_DeviceSetting.trigger_setting.delay / 100));

	copiedValues = ps3000_get_streaming_values(g_Handle, &ClientAreaStartTime,
		pbuffer_a_max,
		pbuffer_a_min,
		pbuffer_b_max,
		pbuffer_b_min,
		pbuffer_c_max,
		pbuffer_c_min,
		pbuffer_d_max,
		pbuffer_d_min,
		&g_overflow, 
		triggerAt,
		triggered,
		no_of_values,
		(unsigned long)denominator);


	if(copiedValues)
	{
		g_TimeFromStart = start_time;
		if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_A].enabled)
		{
			memcpy(display->buffer_max_a, pbuffer_a_max, copiedValues * sizeof(short));
			memcpy(display->buffer_min_a, pbuffer_a_min, copiedValues * sizeof(short));
		}

		if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_B].enabled)
		{
			memcpy(display->buffer_max_b, pbuffer_b_max, copiedValues * sizeof(short));
			memcpy(display->buffer_min_b, pbuffer_b_min, copiedValues * sizeof(short));
		}
		if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_C].enabled)
		{
			memcpy(display->buffer_max_c, pbuffer_c_max, copiedValues * sizeof(short));
			memcpy(display->buffer_min_c, pbuffer_c_min, copiedValues * sizeof(short));
		}
		if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_D].enabled)
		{
			memcpy(display->buffer_max_d, pbuffer_d_max, copiedValues * sizeof(short));
			memcpy(display->buffer_min_d, pbuffer_d_min, copiedValues * sizeof(short));
		}
	}
	return  copiedValues;
}


void get_unagregated_data()
{
	short * unagregatedData[2];
	short overflow;
	unsigned long triggerAt;
	short trigger;
	//	double startTime;
	double time_interval_ns;

	switch(g_DeviceSetting.sample_setting.time_unit)
	{
	case PS3000_NS:
		time_interval_ns = g_DeviceSetting.sample_setting.interval;
		break;
	case PS3000_US:
		time_interval_ns = g_DeviceSetting.sample_setting.interval * 1000;
		break;
	case PS3000_MS:
		time_interval_ns = g_DeviceSetting.sample_setting.interval * 1000 * 1000;
		break;
	default:
		time_interval_ns = g_DeviceSetting.sample_setting.interval;
	}


	unagregatedData[0] = new short[g_DeviceSetting.sample_setting.no_of_samples];
	unagregatedData[1] = new short[g_DeviceSetting.sample_setting.no_of_samples];


	double ClientAreaStartTime = 0;
	if(g_DeviceSetting.trigger_setting.enabled)
		ClientAreaStartTime =  time_interval_ns * ((double)g_DeviceSetting.sample_setting.no_of_samples * ((double)g_DeviceSetting.trigger_setting.delay / 100));



	g_recieved = ps3000_get_streaming_values_no_aggregation(g_Handle,
		&ClientAreaStartTime,
		unagregatedData[0],
		unagregatedData[1],
		NULL,
		NULL,
		&overflow,
		&triggerAt,
		&trigger,
		g_DeviceSetting.sample_setting.no_of_samples);
	if(g_recieved)
		g_bRecieved = true;

	// to be completed save the data to disk

	delete [] unagregatedData[0];
	delete [] unagregatedData[1];
}

long get_data(long x1, long x2, WINDOW_DISPLAY * display, unsigned long *triggerAt, short *triggered, RECT clientArea)
{
	unsigned long copiedValues;

	short pbuffer_a_min[MAX_SAMPLES];
	short pbuffer_a_max[MAX_SAMPLES];
	short pbuffer_b_min[MAX_SAMPLES];
	short pbuffer_b_max[MAX_SAMPLES];
	short pbuffer_c_min[MAX_SAMPLES];
	short pbuffer_c_max[MAX_SAMPLES];
	short pbuffer_d_min[MAX_SAMPLES];
	short pbuffer_d_max[MAX_SAMPLES];
	unsigned long no_of_values = MAX_SAMPLES;
	double	start_time, stop_time;
	short overflow = 0;
	double ratio;
	double time_interval_ns;
	double timeAdjust;

	long low = __min(x1, x2);
	long high = __max(x1, x2);
	switch(g_DeviceSetting.sample_setting.time_unit)
	{
	case PS3000_NS:
		time_interval_ns = g_DeviceSetting.sample_setting.interval;
		break;
	case PS3000_US:
		time_interval_ns = g_DeviceSetting.sample_setting.interval * 1000;
		break;
	case PS3000_MS:
		time_interval_ns = g_DeviceSetting.sample_setting.interval * 1000 * 1000;
		break;
	default:
		time_interval_ns = g_DeviceSetting.sample_setting.interval;
	}

	double ClientAreaTime =  denominator * (double)MAX_SAMPLES * ((double)time_interval_ns);
	// calculate the new aggregation ratio
	timeAdjust = start_time = g_TimeFromStart + (((double)low * 100.0) / (double)(clientArea.right - clientArea.left) * ClientAreaTime /100.0);
	stop_time  = (double)g_TimeFromStart + (((double)high * 100.0) / (double)(clientArea.right - clientArea.left)) * ClientAreaTime / 100.0;

	double ClientAreaStartTime = 0;
	if(g_DeviceSetting.trigger_setting.enabled)
		ClientAreaStartTime =  g_DeviceSetting.sample_setting.denominator * time_interval_ns * ((double)MAX_SAMPLES * ((double)g_DeviceSetting.trigger_setting.delay / 100));

	copiedValues = 0;
	ratio = (stop_time - start_time) / ((double)MAX_SAMPLES * (double)time_interval_ns) + 0.5;
	start_time += ClientAreaStartTime;
	if((unsigned long)ratio)
	{
		numerator = 1;
		denominator = (unsigned long)ratio;

		copiedValues = ps3000_get_streaming_values(g_Handle, &start_time,
			pbuffer_a_max,
			pbuffer_a_min,
			pbuffer_b_max,
			pbuffer_b_min,
			pbuffer_c_max,
			pbuffer_c_min,
			pbuffer_d_max,
			pbuffer_d_min,
			&g_overflow,
			triggerAt,
			triggered,
			no_of_values,
			(unsigned long)denominator);


		if(copiedValues)
		{
			g_TimeFromStart = start_time - ClientAreaStartTime;

			if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_A].enabled)
			{
				memcpy(display->buffer_max_a, pbuffer_a_max, copiedValues * sizeof(short));
				memcpy(display->buffer_min_a, pbuffer_a_min, copiedValues * sizeof(short));
			}

			if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_B].enabled)
			{
				memcpy(display->buffer_max_b, pbuffer_b_max, copiedValues * sizeof(short));
				memcpy(display->buffer_min_b, pbuffer_b_min, copiedValues * sizeof(short));
			}

			if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_C].enabled)
			{
				memcpy(display->buffer_max_c, pbuffer_c_max, copiedValues * sizeof(short));
				memcpy(display->buffer_min_c, pbuffer_c_min, copiedValues * sizeof(short));
			}

			if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_D].enabled)
			{
				memcpy(display->buffer_max_d, pbuffer_d_max, copiedValues * sizeof(short));
				memcpy(display->buffer_min_d, pbuffer_d_min, copiedValues * sizeof(short));
			}
		}
	}

	return  copiedValues;
}

int APIENTRY _tWinMain(HINSTANCE hInstance,
					   HINSTANCE hPrevInstance,
					   LPTSTR    lpCmdLine,
					   int       nCmdShow)
{
	// TODO: Place code here.
	MSG msg;
	HACCEL hAccelTable;

	g_TimeFromStart = 0;
	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_STREAMINGTESTS, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_STREAMINGTESTS);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	save_config_settings();
	return (int) msg.wParam;
}

ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_STREAMINGTESTS);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= (LPCTSTR)IDC_STREAMINGTESTS;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

	return RegisterClassEx(&wcex);
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	HWND hWnd;

	ZeroMemory(&g_DeviceSetting, sizeof(DEVICE_SETTINGS));

	load_config_settings();

	hInst = hInstance;

	hWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

	if (!hWnd)
	{
		return FALSE;
	}

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	return TRUE;
}

void draw_zoom_box(HDC hdc, long x1, long y1, long x2, long y2)
{
	Rectangle(hdc, x1, y1, x2, y2);
}

void save_config_settings(void)
{
	char str[20];

	short entryPoint = 0;
	short keyname = 0;

	sprintf_s(str, sizeof(str) / sizeof(str[0]), "%d",g_DeviceSetting.trigger_setting.enabled);
	WritePrivateProfileString(szSection[entryPoint], szTrigger[keyname++], str, STREAMING_CFG);

	sprintf_s(str, sizeof(str) / sizeof(str[0]), "%d", g_DeviceSetting.trigger_setting.channel);
	WritePrivateProfileString(szSection[entryPoint], szTrigger[keyname++], str, STREAMING_CFG);

	sprintf_s(str, sizeof(str) / sizeof(str[0]), "%d", g_DeviceSetting.trigger_setting.direction);
	WritePrivateProfileString(szSection[entryPoint], szTrigger[keyname++], str, STREAMING_CFG);

	sprintf_s(str, sizeof(str) / sizeof(str[0]), "%d", g_DeviceSetting.trigger_setting.threshold);
	WritePrivateProfileString(szSection[entryPoint], szTrigger[keyname++], str, STREAMING_CFG);

	sprintf_s(str, sizeof(str) / sizeof(str[0]), "%d", g_DeviceSetting.trigger_setting.delay);
	WritePrivateProfileString(szSection[entryPoint], szTrigger[keyname++], str, STREAMING_CFG);

	sprintf_s(str, sizeof(str) / sizeof(str[0]), "%d", g_DeviceSetting.trigger_setting.auto_trigger);
	WritePrivateProfileString(szSection[entryPoint], szTrigger[keyname++], str, STREAMING_CFG);

	sprintf_s(str, sizeof(str) / sizeof(str[0]), "%d", g_DeviceSetting.trigger_setting.auto_trigger_time);
	WritePrivateProfileString(szSection[entryPoint], szTrigger[keyname], str, STREAMING_CFG);

	entryPoint++;
	char str2[20];
	// get the channel details
	for (int i = 0; i < PS3000_MAX_CHANNELS; i++)
	{
		keyname = 0;
		sprintf_s(str, sizeof(str) / sizeof(str[0]), "%d", g_DeviceSetting.channel_Setting[i].enabled);
		sprintf_s(str2, sizeof(str2) / sizeof(str2[0]),  szChannel[keyname++], i);
		WritePrivateProfileString(szSection[entryPoint], str2, str, STREAMING_CFG);

		sprintf_s(str, sizeof(str) / sizeof(str[0]), "%d", g_DeviceSetting.channel_Setting[i].range);
		sprintf_s(str2, sizeof(str2) / sizeof(str2[0]),  szChannel[keyname++], i);
		WritePrivateProfileString(szSection[entryPoint], str2, str, STREAMING_CFG);

		sprintf_s(str, sizeof(str) / sizeof(str[0]), "%d", g_DeviceSetting.channel_Setting[i].coupling);
		sprintf_s(str2, sizeof(str2) / sizeof(str2[0]),  szChannel[keyname], i);
		WritePrivateProfileString(szSection[entryPoint], str2, str, STREAMING_CFG);
	}


	// get the sample details
	entryPoint++;
	keyname = 0;
	sprintf_s(str, sizeof(str) / sizeof(str[0]), "%u", g_DeviceSetting.sample_setting.interval);
	WritePrivateProfileString(szSection[entryPoint], szSample[keyname++], str, STREAMING_CFG);


	sprintf_s(str, sizeof(str) / sizeof(str[0]), "%u", g_DeviceSetting.sample_setting.time_unit);
	WritePrivateProfileString(szSection[entryPoint], szSample[keyname++], str, STREAMING_CFG);

	sprintf_s(str, sizeof(str) / sizeof(str[0]), "%u", g_DeviceSetting.sample_setting.no_of_samples);
	WritePrivateProfileString(szSection[entryPoint], szSample[keyname++], str, STREAMING_CFG);

	sprintf_s(str, sizeof(str) / sizeof(str[0]), "%u", g_DeviceSetting.sample_setting.numerator);
	WritePrivateProfileString(szSection[entryPoint], szSample[keyname++], str, STREAMING_CFG);

	sprintf_s(str, sizeof(str) / sizeof(str[0]), "%u", g_DeviceSetting.sample_setting.denominator);
	WritePrivateProfileString(szSection[entryPoint], szSample[keyname++], str, STREAMING_CFG);

	sprintf_s(str, sizeof(str) / sizeof(str[0]), "%d", g_DeviceSetting.sample_setting.auto_stop);
	WritePrivateProfileString(szSection[entryPoint], szSample[keyname], str, STREAMING_CFG);
}

void load_config_settings(void)
{
	char returnedString[MAX_CHARS];

	short entryPoint = 0;
	short keyname = 0;

	// get the trigger details
	GetPrivateProfileString(szSection[entryPoint], szTrigger[keyname++], "0", returnedString, MAX_CHARS, STREAMING_CFG);
	g_DeviceSetting.trigger_setting.enabled = (short) atoi(returnedString);

	GetPrivateProfileString(szSection[entryPoint], szTrigger[keyname++], "0", returnedString, MAX_CHARS, STREAMING_CFG);
	g_DeviceSetting.trigger_setting.channel = (PS3000_CHANNEL) atoi(returnedString);

	GetPrivateProfileString(szSection[entryPoint], szTrigger[keyname++], "0", returnedString, MAX_CHARS, STREAMING_CFG);
	g_DeviceSetting.trigger_setting.direction = (PS3000_TDIR)atoi(returnedString);

	GetPrivateProfileString(szSection[entryPoint], szTrigger[keyname++], "0", returnedString, MAX_CHARS, STREAMING_CFG);
	g_DeviceSetting.trigger_setting.threshold = (short)atoi(returnedString);

	GetPrivateProfileString(szSection[entryPoint], szTrigger[keyname++], "0", returnedString, MAX_CHARS, STREAMING_CFG);
	g_DeviceSetting.trigger_setting.delay = (short)atoi(returnedString);

	GetPrivateProfileString(szSection[entryPoint], szTrigger[keyname++], "0", returnedString, MAX_CHARS, STREAMING_CFG);
	g_DeviceSetting.trigger_setting.auto_trigger = (short)atoi(returnedString);

	GetPrivateProfileString(szSection[entryPoint], szTrigger[keyname], "1000", returnedString, MAX_CHARS, STREAMING_CFG);
	g_DeviceSetting.trigger_setting.auto_trigger_time = (short)atoi(returnedString);


	entryPoint++;
	char str[20];
	// get the channel details
	for (int i = 0; i < PS3000_MAX_CHANNELS; i++)
	{
		keyname = 0;
		sprintf_s(str, sizeof(str) / sizeof(str[0]), szChannel[keyname++], i);
		GetPrivateProfileString(szSection[entryPoint], str, "0", returnedString, MAX_CHARS, STREAMING_CFG);
		g_DeviceSetting.channel_Setting[i].enabled = (short)atoi(returnedString);

		sprintf_s(str, sizeof(str) / sizeof(str[0]), szChannel[keyname++], i);
		GetPrivateProfileString(szSection[entryPoint], str, "0", returnedString, MAX_CHARS, STREAMING_CFG);
		g_DeviceSetting.channel_Setting[i].range = (PS3000_RANGE)atoi(returnedString);

		sprintf_s(str, sizeof(str) / sizeof(str[0]), szChannel[keyname], i);
		GetPrivateProfileString(szSection[entryPoint], str, "1", returnedString, MAX_CHARS, STREAMING_CFG);
		g_DeviceSetting.channel_Setting[i].coupling = (short)atoi(returnedString);
	}

	// get the sample details
	entryPoint++;
	keyname = 0;
	GetPrivateProfileString(szSection[entryPoint], szSample[keyname++], "600", returnedString, MAX_CHARS, STREAMING_CFG);
	g_DeviceSetting.sample_setting.interval = (unsigned long)atol(returnedString);

	GetPrivateProfileString(szSection[entryPoint], szSample[keyname++], "0", returnedString, MAX_CHARS, STREAMING_CFG);
	g_DeviceSetting.sample_setting.time_unit = (PS3000_TIME_UNITS)atoi(returnedString);

	GetPrivateProfileString(szSection[entryPoint], szSample[keyname++], "1000000", returnedString, MAX_CHARS, STREAMING_CFG);
	g_DeviceSetting.sample_setting.no_of_samples = (unsigned long)atol(returnedString);

	GetPrivateProfileString(szSection[entryPoint], szSample[keyname++], "1", returnedString, MAX_CHARS, STREAMING_CFG);
	g_DeviceSetting.sample_setting.numerator = (unsigned long)atol(returnedString);

	GetPrivateProfileString(szSection[entryPoint], szSample[keyname++], "1000", returnedString, MAX_CHARS, STREAMING_CFG);
	g_DeviceSetting.sample_setting.denominator = (unsigned long)atol(returnedString);

	GetPrivateProfileString(szSection[entryPoint], szSample[keyname], "0", returnedString, MAX_CHARS, STREAMING_CFG);
	g_DeviceSetting.sample_setting.auto_stop = (short)atoi(returnedString);
}


BOOL initialize_device(void)
{
	short triggerChannel;
	short  adcThreshold;
	short autoTrigger;

	g_StopFlag = 0;
	g_TimeFromStart = 0;
	g_GrandNumberOfSamples = 0;
	g_NumberOfSamples = 0;
	g_overflow = 0;
	g_recieved = 0;
	g_bRecieved = false;

	numerator = g_DeviceSetting.sample_setting.numerator;
	denominator = g_DeviceSetting.sample_setting.denominator;
	if(g_DeviceSetting.trigger_setting.enabled)
		adcThreshold = volt_to_adc(g_DeviceSetting.trigger_setting.threshold, 
		g_DeviceSetting.channel_Setting[g_DeviceSetting.trigger_setting.channel].range); 
	else
		adcThreshold = 0;

	triggerChannel = (short)(g_DeviceSetting.trigger_setting.enabled ? g_DeviceSetting.trigger_setting.channel : PS3000_NONE);
	autoTrigger = (short)(g_DeviceSetting.trigger_setting.auto_trigger ? g_DeviceSetting.trigger_setting.auto_trigger_time : 0);

	// initialise the channels
	for (int i = 0; i < PS3000_MAX_CHANNELS; i++)
	{
		ps3000_set_channel(g_Handle,
			i,
			g_DeviceSetting.channel_Setting[i].enabled,
			g_DeviceSetting.channel_Setting[i].coupling,
			g_DeviceSetting.channel_Setting[i].range);
	}
	// switch off ets
	ps3000_set_ets ( g_Handle, PS3000_ETS_OFF, 0, 0 );
	// set the trigger levels

	ps3000_set_trigger(g_Handle,
		triggerChannel,
		(short)adcThreshold,
		g_DeviceSetting.trigger_setting.direction,
		g_DeviceSetting.trigger_setting.delay,
		autoTrigger);

	return ps3000_run_streaming_ns(g_Handle,
		g_DeviceSetting.sample_setting.interval,
		g_DeviceSetting.sample_setting.time_unit,
		g_DeviceSetting.sample_setting.no_of_samples,
		g_DeviceSetting.sample_setting.auto_stop,
		(unsigned long)denominator,
		18349);
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;
	PAINTSTRUCT ps;
	HDC hdc;
	RECT rect;
	int x, y;

	static unsigned long recordNumber;
	static UINT_PTR Timer;
	static BOOL streaming;
	static BOOL readyToCollect;
	static unsigned long firstRecord;
	static unsigned long lastRecord;
	static long copiedValues;
	static short state;
	static long mouse_x1; 
	static long mouse_x2;
	static long mouse_y1;
	static long mouse_y2;


	static short pbuffer_a_min[MAX_SAMPLES];
	static short pbuffer_a_max[MAX_SAMPLES];
	static short pbuffer_b_min[MAX_SAMPLES];
	static short pbuffer_b_max[MAX_SAMPLES];
	static short pbuffer_c_min[MAX_SAMPLES];
	static short pbuffer_c_max[MAX_SAMPLES];
	static short pbuffer_d_min[MAX_SAMPLES];
	static short pbuffer_d_max[MAX_SAMPLES];

	short trigger;
	unsigned long triggerAt;
	short auto_stop;

	static unsigned long no_of_values = MAX_SAMPLES;
	unsigned long start_time = 0;
	short overflow = 0;
	static BOOL bLButtonPressed = 0;
	static BOOL bClearRect = 1;

	switch (message) 
	{
	case WM_TIMER:
		if(g_Handle && streaming)
		{
			if(!readyToCollect)
			{
				readyToCollect = ps3000_ready(g_Handle);
			}
			else
			{
				trigger = FALSE;
				auto_stop = FALSE;
				long noOfValues;
				noOfValues = ps3000_get_streaming_last_values(g_Handle,
					(GetOverviewBuffersMaxMin)StreamingGetOverviewBuffersMaxMin);
				if(g_StopFlag)
				{
					PostMessage(g_hWnd, WM_COMMAND, ID_STREAMING_STOP, 0);
				}
			}
		}
		break;
	case WM_CREATE:
		g_hWnd = hWnd;
		trigger = FALSE;
		copiedValues = 0;
		lastRecord = firstRecord = 1;
		streaming = FALSE;
		g_Handle = FALSE;
		Timer = SetTimer(hWnd, NULL, TIMER_INTERUPT, NULL); 
		g_trigger = 0;
		g_triggerIndex = 0;
		break;

	case WM_CHAR:
		switch (wParam)
		{
		case ' ':
			if(streaming)
				SendMessage(hWnd, WM_COMMAND, ID_STREAMING_STOP, 0);
			else
				SendMessage(hWnd, WM_COMMAND, ID_STREAMING_START, 0);
			break;

		case 'O':
		case 'o':
			SendMessage(hWnd, WM_COMMAND, ID_FILE_OPENDEVICE, 0);
			break;


		case 'C':  
		case 'c':
			SendMessage(hWnd, WM_COMMAND, ID_SETTINGS_CHANNEL, 0);
			break;

		case 'T':
		case 't':
			SendMessage(hWnd, WM_COMMAND, ID_SETTINGS_TRIGGER, 0);
			break;

		case 'S':
		case 's':
			SendMessage(hWnd, WM_COMMAND, ID_SETTINGS_SAMPLING, 0);
			break;
		}
		break;

	case WM_MOUSEMOVE:
		if(!streaming)
		{
			if(wParam == MK_LBUTTON)
			{
				if(!bLButtonPressed)
				{
					mouse_x1 = GET_X_LPARAM(lParam); 
					mouse_y1 = GET_Y_LPARAM(lParam); 
					bLButtonPressed = TRUE;
					bClearRect = FALSE;
				}

				mouse_x2 = GET_X_LPARAM(lParam); 
				mouse_y2 = GET_Y_LPARAM(lParam); 
				InvalidateRect(hWnd, NULL, TRUE);
			}

			if(bLButtonPressed && wParam != MK_LBUTTON)
			{
				// Get the new values
				GetClientRect(hWnd, &rect);
				if(get_data(mouse_x1, mouse_x2, &g_display, &triggerAt, &g_trigger, rect))
				{
					if(g_trigger)
						g_triggerIndex = triggerAt;
				}
				bClearRect = TRUE;
				bLButtonPressed = FALSE;
				InvalidateRect(hWnd, NULL, TRUE);
			}
		}
		break;

	case WM_COMMAND:
		wmId    = LOWORD(wParam); 
		wmEvent = HIWORD(wParam); 
		// Parse the menu selections:
		switch (wmId)
		{
		case ID_FILE_OPENDEVICE:
			if(!g_Handle)
				g_Handle = ps3000_open_unit();
			break;

		case ID_STREAMING_STOP:
			if(g_Handle && streaming)
			{
				streaming = FALSE;
				ps3000_stop(g_Handle);
				get_unagregated_data();
				InvalidateRect(hWnd, NULL, TRUE);
			}
			break;
		case ID_STREAMING_START:
			if(g_Handle && !streaming)
			{
				g_trigger = 0;
				g_triggerIndex = 0;
				streaming = initialize_device();
				readyToCollect = FALSE;
			}
			break;
		case ID_STREAMING_RESET: // display the original results
			reset_data(&g_display, &g_triggerIndex, &g_trigger);
			InvalidateRect(hWnd, NULL, FALSE);
			break;
		case ID_STREAMING_UNAGGREGATED: // display unagreagated data
			get_unagregated_data();
			break;

		case ID_SETTINGS_TRIGGER:
			if(DialogBox(hInst, MAKEINTRESOURCE(IDD_TRIGGER), hWnd, TriggerDlgProc))
				SendMessage(hWnd, WM_COMMAND, ID_STREAMING_STOP, 0);
			break;

		case ID_SETTINGS_CHANNEL:
			if(DialogBox(hInst, MAKEINTRESOURCE(IDD_CHANNEL_SETTINGS), hWnd, ChannelDlgProc))
				SendMessage(hWnd, WM_COMMAND, ID_STREAMING_STOP, 0);
			break;

		case ID_SETTINGS_SAMPLING:
			if(DialogBox(hInst, MAKEINTRESOURCE(IDD_SAMPLE), hWnd, SamplingDlgProc))
				SendMessage(hWnd, WM_COMMAND, ID_STREAMING_STOP, 0);
			break;

		case IDM_ABOUT:
			DialogBox(hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
			break;
		case IDM_EXIT:
			if(g_Handle)
			{
				if(streaming)
					ps3000_stop(g_Handle);
				ps3000_close_unit(g_Handle);
				streaming = FALSE;
				g_Handle = 0;
			}
			DestroyWindow(hWnd);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
		}
		break;
	case WM_PAINT:
		hdc = BeginPaint(hWnd, &ps);
		GetClientRect(hWnd, &rect);
		Rectangle(hdc, rect.left, rect.top, rect.right, rect.bottom);

		if(!streaming && !bClearRect)
		{
			draw_zoom_box(hdc, mouse_x1, mouse_y1, mouse_x2, mouse_y2);
		}

		x = (int) rect.right - FROM_RIGHT;
		y = (int) rect.top + FROM_TOP;
		draw_count_text(hdc, x, y);


		draw_grid(hdc, rect);
		if(g_NumberOfSamples)
		{
			COLORREF colRef[] = {0x000000FF, 0x0000FF00, 0x00FF0000, 0x00EE00EE};

			if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_A].enabled)
				draw_data_line(hdc, g_display.buffer_max_a, g_display.buffer_min_a, MAX_SAMPLES, rect, colRef[PS3000_CHANNEL_A]);

			if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_B].enabled)
				draw_data_line(hdc, g_display.buffer_max_b, g_display.buffer_min_b, MAX_SAMPLES, rect, colRef[PS3000_CHANNEL_B]);

			if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_C].enabled)
				draw_data_line(hdc, g_display.buffer_max_c, g_display.buffer_min_c, MAX_SAMPLES, rect, colRef[PS3000_CHANNEL_C]);

			if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_D].enabled)
				draw_data_line(hdc, g_display.buffer_max_d, g_display.buffer_min_d, MAX_SAMPLES, rect, colRef[PS3000_CHANNEL_D]);

			if(g_trigger)
				draw_trigger_point(hdc, rect, g_triggerIndex);
		}

		EndPaint(hWnd, &ps);
		break;
	case WM_DESTROY:
		if(g_Handle)
		{
			if(streaming)
				ps3000_stop(g_Handle);
			Sleep(50);
			ps3000_close_unit(g_Handle);
		}
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

// Message handler for about box.
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		return TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
		{
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		break;
	}
	return FALSE;
}

static INT_PTR CALLBACK TriggerDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
#define MAX_DIRECTIONS 2
	char *szDirection[] = {"Rising","Falling"};
	char *szChannel[] = {"OFF", "ChA", "ChB", "ChC", "ChD"};
	int channelSelected;
	BOOL translated;

	switch (message)
	{
	case WM_INITDIALOG:
		for (int i = 0; i <MAX_DIRECTIONS; i++)
		{
			SendDlgItemMessage(hWnd, IDC_DIRECTION, CB_ADDSTRING, 0, (LPARAM) (LPCTSTR) szDirection[i]);
		}

		for (int i = 0; i <PS3000_MAX_CHANNELS + 1; i++)
		{
			SendDlgItemMessage(hWnd, IDC_CHANNEL, CB_ADDSTRING, 0, (LPARAM) (LPCTSTR) szChannel[i]);
		}

		SendDlgItemMessage (hWnd, IDC_AUTO_TRIGGER, BM_SETCHECK, g_DeviceSetting.trigger_setting.auto_trigger, 0);
		SendDlgItemMessage(hWnd, IDC_CHANNEL, CB_SETCURSEL, (int)g_DeviceSetting.trigger_setting.channel + 1, 0);
		SendDlgItemMessage(hWnd, IDC_DIRECTION, CB_SETCURSEL, (int)g_DeviceSetting.trigger_setting.direction, 0);
		SetDlgItemInt(hWnd, IDC_AUTO_TIGGER_TIME, g_DeviceSetting.trigger_setting.auto_trigger_time, FALSE);
		SetDlgItemInt(hWnd, IDC_DELAY, g_DeviceSetting.trigger_setting.delay, TRUE);
		SetDlgItemInt(hWnd, IDC_THRESHOLD, g_DeviceSetting.trigger_setting.threshold, TRUE);
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
			channelSelected = (int)SendDlgItemMessage(hWnd, IDC_CHANNEL, CB_GETCURSEL, 0, 0);
			g_DeviceSetting.trigger_setting.enabled = (short)(channelSelected > 0);
			g_DeviceSetting.trigger_setting.channel = (PS3000_CHANNEL)(channelSelected - 1);
			g_DeviceSetting.trigger_setting.direction = (PS3000_TDIR)SendDlgItemMessage(hWnd, IDC_DIRECTION, CB_GETCURSEL, 0, 0);
			g_DeviceSetting.trigger_setting.threshold = GetDlgItemInt(hWnd, IDC_THRESHOLD, &translated, TRUE);
			g_DeviceSetting.trigger_setting.auto_trigger = (short)SendDlgItemMessage (hWnd, IDC_AUTO_TRIGGER, BM_GETCHECK, 0, 0);
			g_DeviceSetting.trigger_setting.auto_trigger_time = GetDlgItemInt(hWnd, IDC_AUTO_TIGGER_TIME, &translated, FALSE);
			g_DeviceSetting.trigger_setting.delay = (short)GetDlgItemInt(hWnd, IDC_DELAY, &translated, TRUE);
			EndDialog(hWnd, TRUE);
			return TRUE;

		case IDCANCEL:
			EndDialog(hWnd, FALSE);
			return TRUE;
		}
		break;
	}

	return FALSE;
}

static INT_PTR CALLBACK ChannelDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
#define MAX_VOLTAGE_RANGES 11
	char *szVoltage[] = {"Off","20mV","50mV","100mV","200mV","500mV","1V","2V","5V","10V","20V"};
	int voltageSelected;

#define MAX_COUPLING 2
	char *szCoupling[] = {"AC", "DC"};

	switch (message)
	{
	case WM_INITDIALOG:
		for(int ch = 0; ch < PS3000_MAX_CHANNELS; ch++)
		{
			for (int i = 0; i <MAX_VOLTAGE_RANGES; i++)
			{
				SendDlgItemMessage(hWnd, IDC_CHA_RANGE + ch, CB_ADDSTRING, 0, (LPARAM) (LPCTSTR) szVoltage[i]);
			}

			for (int i = 0; i <MAX_COUPLING; i++)
			{
				SendDlgItemMessage(hWnd, IDC_CHA_COUPLING + ch, CB_ADDSTRING, 0, (LPARAM) (LPCTSTR) szCoupling[i]);
			}

			SendDlgItemMessage(hWnd, IDC_CHA_RANGE + ch, CB_SETCURSEL, (int)g_DeviceSetting.channel_Setting[ch].range, 0);
			SendDlgItemMessage(hWnd, IDC_CHA_COUPLING + ch, CB_SETCURSEL, g_DeviceSetting.channel_Setting[ch].coupling, 0);
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
			for (int ch = 0; ch < PS3000_MAX_CHANNELS; ch++)
			{
				voltageSelected = (int)SendDlgItemMessage(hWnd, IDC_CHA_RANGE + ch, CB_GETCURSEL, 0, 0);
				g_DeviceSetting.channel_Setting[ch].enabled = voltageSelected > 0;
				g_DeviceSetting.channel_Setting[ch].range = (PS3000_RANGE)(g_DeviceSetting.channel_Setting[ch].enabled ? (voltageSelected) : PS3000_10MV);
				g_DeviceSetting.channel_Setting[ch].coupling = (short)SendDlgItemMessage(hWnd, IDC_CHA_COUPLING + ch, CB_GETCURSEL, 0, 0);
			}
			EndDialog(hWnd, TRUE);
			return TRUE;

		case IDCANCEL:
			EndDialog(hWnd, FALSE);
			return TRUE;
		}
		break;
	}

	return FALSE;

}

static INT_PTR CALLBACK SamplingDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
#define MAX_UNITS 4
	char *szUnits[] = {"Nano","Micro", "Milli", "Sec"};
	unsigned long currentSetting;
	BOOL translated;

	switch (message)
	{
	case WM_INITDIALOG:
		for (int i = 0; i <MAX_UNITS; i++)
		{
			SendDlgItemMessage(hWnd, IDC_UNITS, CB_ADDSTRING, 0, (LPARAM) (LPCTSTR) szUnits[i]);
		}

		SendDlgItemMessage(hWnd, IDC_UNITS, CB_SETCURSEL, (int)g_DeviceSetting.sample_setting.time_unit - 2, 0);
		SendDlgItemMessage (hWnd, IDC_AUTO_STOP, BM_SETCHECK, (int)g_DeviceSetting.sample_setting.auto_stop, 0);

		currentSetting = g_DeviceSetting.sample_setting.interval == 0 ? 600 : (int)g_DeviceSetting.sample_setting.interval;
		SetDlgItemInt(hWnd, IDC_INTERVAL, currentSetting, FALSE);

		currentSetting = (int)g_DeviceSetting.sample_setting.no_of_samples == 0 ? 1000000 : g_DeviceSetting.sample_setting.no_of_samples; 
		SetDlgItemInt(hWnd, IDC_NO_OF_SAMPLES, currentSetting, TRUE);

		currentSetting = (int)g_DeviceSetting.sample_setting.numerator == 0 ? 1 : g_DeviceSetting.sample_setting.numerator;
		SetDlgItemInt(hWnd, IDC_NUMERATOR, currentSetting, TRUE);

		currentSetting = (int)g_DeviceSetting.sample_setting.denominator  == 0 ? 1000 : g_DeviceSetting.sample_setting.denominator;
		SetDlgItemInt(hWnd, IDC_DENOMINATOR, currentSetting, TRUE);
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDOK:
			g_DeviceSetting.sample_setting.time_unit = (PS3000_TIME_UNITS)(SendDlgItemMessage(hWnd, IDC_UNITS, CB_GETCURSEL, 0, 0) + 2);
			g_DeviceSetting.sample_setting.auto_stop = (short)SendDlgItemMessage (hWnd, IDC_AUTO_STOP, BM_GETCHECK, 0, 0);
			g_DeviceSetting.sample_setting.interval = GetDlgItemInt(hWnd, IDC_INTERVAL, &translated, FALSE);
			g_DeviceSetting.sample_setting.no_of_samples = GetDlgItemInt(hWnd, IDC_NO_OF_SAMPLES, &translated, FALSE);
			g_DeviceSetting.sample_setting.numerator = GetDlgItemInt(hWnd, IDC_NUMERATOR, &translated, FALSE);
			g_DeviceSetting.sample_setting.denominator = GetDlgItemInt(hWnd, IDC_DENOMINATOR, &translated, FALSE);
			EndDialog(hWnd, TRUE);
			return TRUE;

		case IDCANCEL:
			EndDialog(hWnd, FALSE);
			return TRUE;
		}
		break;
	}

	return FALSE;

}

short ps3000_callback_func (short handle, short *dataBuffer, short noOfBuffers)
{
	return 1;
}

void StreamingGetOverviewBuffersMaxMin (
										short **overviewBuffers,
										short overflow,
										unsigned long triggeredAt,
										short triggered,
										short auto_stop,
										unsigned long nValues)
{
	if(nValues)
	{
		if(g_DeviceSetting.trigger_setting.enabled)
		{
			if(g_trigger)
				g_GrandNumberOfSamples += nValues;
			else if(triggered)
				g_GrandNumberOfSamples += (nValues - triggeredAt);
		}
		else
		{
			g_GrandNumberOfSamples += nValues;
		}
		if(!g_overflow)
			g_overflow |= overflow;
		if(nValues > MAX_SAMPLES)
			nValues = MAX_SAMPLES;
		short free = (short)(MAX_SAMPLES - g_NumberOfSamples);
		g_NumberOfSamples += nValues;
		if(g_NumberOfSamples > MAX_SAMPLES)
			g_NumberOfSamples = MAX_SAMPLES;

		if(g_NumberOfSamples == MAX_SAMPLES)
		{
			if(g_trigger)
			{
				g_triggerIndex = (g_triggerIndex - nValues ) + free;
			}
			if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_A].enabled)
			{
				memcpy(g_display.buffer_max_a, (g_display.buffer_max_a + nValues) - free, ((g_NumberOfSamples - nValues) + free) * sizeof(short));
				memcpy(g_display.buffer_min_a, (g_display.buffer_min_a + nValues) - free, ((g_NumberOfSamples - nValues) + free) * sizeof(short));
			}

			if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_B].enabled)
			{
				memcpy(g_display.buffer_max_b, (g_display.buffer_max_b + nValues) - free, ((g_NumberOfSamples - nValues) + free) * sizeof(short));
				memcpy(g_display.buffer_min_b, (g_display.buffer_min_b + nValues) - free, ((g_NumberOfSamples - nValues) + free) * sizeof(short));
			}

			if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_C].enabled)
			{
				memcpy(g_display.buffer_max_c, (g_display.buffer_max_c + nValues) - free, ((g_NumberOfSamples - nValues) + free) * sizeof(short));
				memcpy(g_display.buffer_min_c, (g_display.buffer_min_c + nValues) - free, ((g_NumberOfSamples - nValues) + free) * sizeof(short));
			}

			if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_D].enabled)
			{
				memcpy(g_display.buffer_max_d, (g_display.buffer_max_d + nValues) - free, ((g_NumberOfSamples - nValues) + free) * sizeof(short));
				memcpy(g_display.buffer_min_d, (g_display.buffer_min_d + nValues) - free, ((g_NumberOfSamples - nValues) + free) * sizeof(short));
			}
		}

		if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_A].enabled)
		{
			memcpy(g_display.buffer_max_a + (g_NumberOfSamples - nValues), overviewBuffers[0], nValues * sizeof(short));
			memcpy(g_display.buffer_min_a + (g_NumberOfSamples - nValues), overviewBuffers[1], nValues * sizeof(short));
		}

		if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_B].enabled)
		{
			memcpy(g_display.buffer_max_b + (g_NumberOfSamples - nValues), overviewBuffers[2], nValues * sizeof(short));
			memcpy(g_display.buffer_min_b + (g_NumberOfSamples - nValues), overviewBuffers[3], nValues * sizeof(short));
		}

		if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_C].enabled)
		{
			memcpy(g_display.buffer_max_c + (g_NumberOfSamples - nValues), overviewBuffers[4], nValues * sizeof(short));
			memcpy(g_display.buffer_min_c + (g_NumberOfSamples - nValues), overviewBuffers[5], nValues * sizeof(short));
		}

		if(g_DeviceSetting.channel_Setting[PS3000_CHANNEL_D].enabled)
		{
			memcpy(g_display.buffer_max_d + (g_NumberOfSamples - nValues), overviewBuffers[6], nValues * sizeof(short));
			memcpy(g_display.buffer_min_d + (g_NumberOfSamples - nValues), overviewBuffers[7], nValues * sizeof(short));
		}

		if(triggered)
		{
			g_triggerIndex = g_NumberOfSamples - (nValues - triggeredAt);
		}

		if(triggered)
			g_trigger = triggered;

		InvalidateRect(g_hWnd, NULL, TRUE);
	}
	if(auto_stop)
	{
		g_StopFlag = TRUE;
	}
}
