/**************************************************************************
 *
 * Filename:    PS3000con.c
 *
 * Copyright:   Pico Technology Limited 2002
 *
 * Author:      MTB
 *
 * Description:
 *   This is a console-mode program that demonstrates how to use the
 *  

 * Examples:
 *    Collect a block of samples immediately
 *    Collect a block of samples when a trigger event occurs
 *    Collect a block using ETS
 *    Collect a stream of data
 *
 * To build this application
 *	set up a project for a 32-bit console mode application
 *	add this file to the project
 *	add PS3000bc.lib to the project (Borland C only)
 *	add PS3000.lib to the project (Microsoft C only)
 *	build the project
 *
 * History:
 *     27Nov02 MTB Created
 *
 * Revision Info: "file %n date %f revision %v"
 *						""
 *
 ***************************************************************************/

#include "windows.h"
#include <conio.h>
#include <stdio.h>

/* Definitions of PS3000 driver routines */
#include "PS3000.h"

#define BUFFER_SIZE 	5000000
#define MAX_CHANNELS 4

#define MIN(a,b) ((a)>(b)?(b):(a))

short  	values_a [BUFFER_SIZE];
short  	values_b [BUFFER_SIZE];
short    overflow;
int		scale_to_mv = 1;

short		channel_mv [PS3000_MAX_CHANNELS];
short		timebase = 8;

long 		sig_gen_frequency = 0;
PS3000_WAVE_TYPES sig_gen_wave = PS3000_SINE;

#define QUAD_SCOPE 4
#define DUAL_SCOPE 2

typedef enum {
	MODEL_NONE = 0,
  MODEL_PS3204 = 3204,
	MODEL_PS3205 = 3205,
	MODEL_PS3206 = 3206,
	MODEL_PS3224 = 3224,
	MODEL_PS3424 = 3424,
} MODEL_TYPE;


typedef struct {
	short DCcoupled;
	short range;
	short enabled;
} CHANNEL_SETTINGS;

typedef struct  {
	short handle;
	MODEL_TYPE model;
  PS3000_RANGE firstRange;
	PS3000_RANGE lastRange;
	BYTE signalGenerator;
	BYTE external;
	short timebases;
	short noOfChannels;
	CHANNEL_SETTINGS channelSettings [MAX_CHANNELS];
	short values [MAX_CHANNELS][5000000];
	PS3000_RANGE triggerRange;
} UNIT_MODEL; 

UNIT_MODEL unitOpened;

void StreamingGetOverviewBuffersMaxMin (
										short **overviewBuffers,
										short overflow,
										unsigned long triggeredAt,
										short triggered,
										short auto_stop,
										unsigned long nValues);





long times[BUFFER_SIZE];

short input_ranges [PS3000_MAX_RANGES] = {10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000, 20000, 50000};


/****************************************************************************
 *
 * adc_units
 *
 ****************************************************************************/
char * adc_units (short time_units)
  {
  time_units++;
  printf ( "time unit:  %d\n", time_units ) ;
  switch ( time_units )
    {
    case 0:
      return "ADC";
    case 1:
      return "fs";
    case 2:
      return "ps";
    case 3:
      return "ns";
    case 4:
      return "us";
    case 5:
      return "ms";
    }
  return "Not Known";
  }

/****************************************************************************
 * adc_to_mv
 *
 * If the user selects scaling to millivolts,
 * Convert an 12-bit ADC count into millivolts
 ****************************************************************************/
int adc_to_mv (long raw, int ch)
  {
  return ( scale_to_mv ) ? ( raw * input_ranges[ch] ) / 32767 : raw;
  }

/****************************************************************************
 * mv_to_adc
 *
 * Convert a millivolt value into a 12-bit ADC count
 *
 *  (useful for setting trigger thresholds)
 ****************************************************************************/
short mv_to_adc (short mv, short ch)
  {
	  return ( ( mv * 32767 ) / input_ranges[ch]==0?1:input_ranges[ch] );
  }

/****************************************************************************
 * set_defaults - restore default settings
 ****************************************************************************/
void set_defaults (void)
{
	int i;

	if (unitOpened.model != MODEL_PS3224 && unitOpened.model != MODEL_PS3424)
	{
    ps3000_set_ets ( unitOpened.handle, PS3000_ETS_OFF, 0, 0 );
	}

	for (i = 0; i < unitOpened.noOfChannels; i++)
	{
    ps3000_set_channel ( unitOpened.handle,
			                   PS3000_CHANNEL_A + i,
												 unitOpened.channelSettings[PS3000_CHANNEL_A + i].enabled ,
												 unitOpened.channelSettings[PS3000_CHANNEL_A + i].DCcoupled ,
												 unitOpened.lastRange);
	}
}

/****************************************************************************
 * Collect_block_immediate
 *  this function demonstrates how to collect a single block of data
 *  from the unit (start collecting immediately)
 ****************************************************************************/

void collect_block_immediate (void)
{
  int		i, j;
  long 	time_interval;
  short 	time_units;
  short 	oversample;
  long 	no_of_samples ;
  FILE 	*fp;
  short 	auto_trigger_ms = 0;
  long 	time_indisposed_ms;
  short 	overflow;
  long 	max_samples;

  printf ( "Collect block immediate...\n" );
  printf ( "Press a key to start\n" );
  getch ();

  set_defaults ();
  no_of_samples = 50000; 
  /* Trigger disabled
   */
  ps3000_set_trigger ( unitOpened.handle, PS3000_NONE, 0, PS3000_RISING, 0, auto_trigger_ms );

  /*  find the maximum number of samples, the time interval (in time_units),
   *		 the most suitable time units, and the maximum oversample at the current timebase
   */
	oversample = 1;
  while (!ps3000_get_timebase ( unitOpened.handle,
                        timebase,
  					      	    no_of_samples,
                        &time_interval,
                        &time_units,
                        oversample,
                        &max_samples))
	  timebase++;										;

  printf ( "timebase: x%hd\toversample:x%hd max_samples:%d\n", timebase, oversample,max_samples );
  /* Start it collecting,
   *  then wait for completion
   */
  ps3000_run_block ( unitOpened.handle, no_of_samples, timebase, oversample, &time_indisposed_ms );
  while ( !ps3000_ready ( unitOpened.handle ) )
    {
    Sleep ( 100 );
    }

  ps3000_stop ( unitOpened.handle );

  /* Should be done now...
   *  get the times (in nanoseconds)
   *   and the values (in ADC counts)
   */
  ps3000_get_times_and_values ( unitOpened.handle, times,
		                            unitOpened.values[PS3000_CHANNEL_A], 
		                            unitOpened.values[PS3000_CHANNEL_B],
									unitOpened.values[PS3000_CHANNEL_C],
									unitOpened.values[PS3000_CHANNEL_D],
									&overflow, time_units, MIN(max_samples,no_of_samples) );

  /* Print out the first 10 readings,
   *  converting the readings to mV if required
   */
  printf ( "First 10 readings\n" );
  printf ( "Value\n" );
  printf ( "(%s)\n", adc_units ( time_units ) );

  for ( i = 0; i < 10; i++ )
    {
			for (j = 0; j < unitOpened.noOfChannels; j++)
			{
			  if(unitOpened.channelSettings[j].enabled)
				{
          printf ( "%d\t", adc_to_mv ( unitOpened.values[j][i], 
			  unitOpened.channelSettings[PS3000_CHANNEL_A].range) );
				}
			}
			printf("\n");
    }

  fp = fopen ( "data.txt","w" );
  for ( i = 0; i < BUFFER_SIZE; i++ )
  {
		fprintf ( fp,"%ld ", times[i]);
		for ( j = 0; j < unitOpened.noOfChannels; j++)
		{
      if(unitOpened.channelSettings[j].enabled)
		  {
        fprintf ( fp, ",%d, %d,", unitOpened.values[j][i],
			  		adc_to_mv ( unitOpened.values[j][i], 
					unitOpened.channelSettings[PS3000_CHANNEL_A + j].range) );
			}
    }
		fprintf(fp, "\n");
	}
  fclose(fp);
}

/****************************************************************************
 * Collect_block_triggered
 *  this function demonstrates how to collect a single block of data from the
 *  unit, when a trigger event occurs.
 ****************************************************************************/

void collect_block_triggered (void)
  {
  int		i, j;
  int		trigger_sample;
  long 	time_interval;
  short 	time_units;
  short 	oversample;
  long 	no_of_samples = BUFFER_SIZE;
  FILE 	*fp;
  short 	auto_trigger_ms = 0;
  long 	time_indisposed_ms;
  short 	overflow;
  int 	threshold_mv =1500;
  long 	max_samples;

  printf ( "Collect block triggered...\n" );
  printf ( "Collects when value rises past %dmV\n", threshold_mv );
  printf ( "Press a key to start...\n" );
  getch ();

  set_defaults ();

  /* Trigger enabled
   * Rising edge
   * Threshold = 100mV
   * 10% pre-trigger  (negative is pre-, positive is post-)
   */
  ps3000_set_trigger ( unitOpened.handle,
		             PS3000_CHANNEL_B,
					 mv_to_adc (threshold_mv, unitOpened.channelSettings[PS3000_CHANNEL_B].range),
					 PS3000_RISING, -10,
					 auto_trigger_ms );


  /*  find the maximum number of samples, the time interval (in time_units),
   *		 the most suitable time units, and the maximum oversample at the current timebase
   */
	oversample = 1;
  while (!ps3000_get_timebase ( unitOpened.handle,
                        timebase,
  					      	    no_of_samples,
                        &time_interval,
                        &time_units,
                        oversample,
                        &max_samples))
	  timebase++;

  /* Start it collecting,
   *  then wait for completion
   */
  ps3000_run_block ( unitOpened.handle, BUFFER_SIZE, timebase, oversample, &time_indisposed_ms );

  printf ( "Waiting for trigger..." );
  printf ( "Press a key to abort\n" );

  while (( !ps3000_ready ( unitOpened.handle )) && ( !kbhit () ))
  {
    Sleep ( 100 );
  }

  if (kbhit ())
  {
    getch ();

    printf ( "data collection aborted\n" );
  }
  else
  {
    ps3000_stop ( unitOpened.handle );

    /* Get the times (in units specified by time_units)
     *  and the values (in ADC counts)
     */
    ps3000_get_times_and_values ( unitOpened.handle,
			                            times,
																	unitOpened.values[PS3000_CHANNEL_A],
																	unitOpened.values[PS3000_CHANNEL_B],
																	unitOpened.values[PS3000_CHANNEL_C],
																	unitOpened.values[PS3000_CHANNEL_D],
																	&overflow, time_units, BUFFER_SIZE );

    /* Print out the first 10 readings,
     *  converting the readings to mV if required
     */
    printf ("Ten readings around trigger\n");
    printf ("Time\tValue\n");
    printf ("(ns)\t(%s)\n", adc_units (time_units));

    /* This calculation is correct for 10% pre-trigger
     */
    trigger_sample = BUFFER_SIZE / 10;

    for (i = trigger_sample - 5; i < trigger_sample + 5; i++)
    {
  		for (j = 0; j < unitOpened.noOfChannels; j++)
			{
			  if(unitOpened.channelSettings[j].enabled)
				{
          printf ( "%d\t", adc_to_mv ( unitOpened.values[j][i], 
			  unitOpened.channelSettings[PS3000_CHANNEL_A].range) );
				}
			}
			printf("\n");
    }
 
    fp = fopen ( "data.txt","w" );
    for ( i = 0; i < BUFFER_SIZE; i++ )
    {
		  fprintf ( fp,"%ld ", times[i]);
		  for ( j = 0; j < unitOpened.noOfChannels; j++)
	  	{
        if(unitOpened.channelSettings[j].enabled)
		    {
          fprintf ( fp, ",%d, %d,", unitOpened.values[j][i],
					adc_to_mv ( unitOpened.values[j][i], 
					unitOpened.channelSettings[PS3000_CHANNEL_A + j].range) );
			  }
      }
		  fprintf(fp, "\n");
	  }
    fclose(fp);
	}
}

/****************************************************************************
 * Collect_block_ets
 *  this function demonstrates how to collect a block of
 *  data using equivalent time sampling (ETS).
 ****************************************************************************/

void collect_block_ets (void)
  {
  int		i;
  int		trigger_sample;
  FILE 	*fp;
  short 	auto_trigger_ms = 0;
  long 	time_indisposed_ms;
  short 	overflow;
  long  	ets_sampletime;

	if (unitOpened.model == MODEL_PS3424 || unitOpened.model == MODEL_PS3224)
	{
		printf("Not available with PS%d\n", unitOpened.model); 
		return;
	}

  printf ( "Collect ETS block...\n" );
  printf ( "Collects when value rises past 100mV\n" );
  printf ( "Press a key to start...\n" );
  getch ();

  set_defaults ();

  /* Trigger enabled
   * Rising edge
   * Threshold = 1500mV
   * 10% pre-trigger  (negative is pre-, positive is post-)
   */
   ps3000_set_trigger ( unitOpened.handle,
		                    PS3000_CHANNEL_B,
							mv_to_adc (1500, unitOpened.channelSettings[PS3000_CHANNEL_B].range),
							PS3000_RISING,
							-10,
							auto_trigger_ms );

  /* Enable ETS in fast mode,
   * the computer will store 20 cycles
   *  but interleave only 10
   */
   ets_sampletime = ps3000_set_ets ( unitOpened.handle, PS3000_ETS_FAST, 10, 100 );
   printf ( "ETS Sample Time is: %ld\n", ets_sampletime );
  /* Start it collecting,
   *  then wait for completion
   */
  ps3000_run_block ( unitOpened.handle, BUFFER_SIZE, timebase, 1, &time_indisposed_ms );

  printf ( "Waiting for trigger..." );
  printf ( "Press a key to abort\n" );

  while ( (!ps3000_ready (unitOpened.handle)) && (!kbhit ()) )
    {
    Sleep (100);
    }

  if ( kbhit () )
    {
    getch ();
    printf ( "data collection aborted\n" );
    }
  else
    {
    ps3000_stop ( unitOpened.handle );
    /* Get the times (in microseconds)
     *  and the values (in ADC counts)
     */
    ps3000_get_times_and_values (unitOpened.handle,
			                           times,
																 unitOpened.values[PS3000_CHANNEL_A],
																 unitOpened.values[PS3000_CHANNEL_B],
																 unitOpened.values[PS3000_CHANNEL_C],
																 unitOpened.values[PS3000_CHANNEL_D],
																 &overflow,
																 4,
																 BUFFER_SIZE);

    /* Print out the first 10 readings,
     *  converting the readings to mV if required
     */

    printf ( "Ten readings around trigger\n" );
    printf ( "(ps)\t(%s)\n", adc_units (4) );

    /* This calculation is correct for 10% pre-trigger
     */
    trigger_sample = BUFFER_SIZE / 10;

    for ( i = trigger_sample - 5; i < trigger_sample + 5; i++ )
      {
      printf ( "%ld\t%d\n", times [i], adc_to_mv (values_b [i], 0) );
      }

    fp = fopen ( "data.txt","w" );
    for ( i = 0; i < BUFFER_SIZE; i++ )
    {
       fprintf ( fp, "%ld, %d, %d\n", times[i], unitOpened.values[PS3000_CHANNEL_B][i], 
	adc_to_mv (unitOpened.values[PS3000_CHANNEL_B][i], unitOpened.channelSettings[PS3000_CHANNEL_B].range) );
    }
    fclose( fp );
    }
  }

/****************************************************************************
 *
 * Collect_streaming
 *  this function demonstrates how to use streaming.
 *
 * In this mode, you can collect data continuously.
 *
 * This example writes data to disk...
 * don't leave it running too long or it will fill your disk up!
 *
 * Each call to ps3000_get_times_and_values returns the readings since the
 * last call
 *
 * The time is in microseconds: it will wrap around at 2^32 (approx 2,000 seconds)
 * if you don't need the time, you can just call ps3000_get_values
 *
 ****************************************************************************/

void collect_streaming (void)
  {
  int		i;
  int		block_no;
  FILE 	*fp;
  int		no_of_values;
  short  overflow;
  int 	ok;

  printf ( "Collect streaming...\n" );
  printf ( "Data is written to disk file (test.out)\n" );
  printf ( "Press a key to start\n" );
  getch ();

  set_defaults ();

  /* You cannot use triggering for the start of the data...
   */
  ps3000_set_trigger ( unitOpened.handle, PS3000_NONE, 0, 0, 0, 0 );

  /* Collect data at 10ms intervals
   * Max BUFFER_SIZE points on each call
   *  (buffer must be big enough for max time between calls
   *
   *  Start it collecting,
   *  then wait for trigger event
   */
  ok = ps3000_run_streaming ( unitOpened.handle, 10, 1000, 0 );
  printf ( "OK: %d\n", ok );

  /* From here on, we can get data whenever we want...
   */
  block_no = 0;
  fp = fopen ( "data.txt", "w" );
  while ( !kbhit () )
    {
    no_of_values = ps3000_get_values ( unitOpened.handle, 
			                                 unitOpened.values[PS3000_CHANNEL_A],
																	 unitOpened.values[PS3000_CHANNEL_B],
																	 unitOpened.values[PS3000_CHANNEL_C],
																	 unitOpened.values[PS3000_CHANNEL_D],
																	 &overflow,
																	 BUFFER_SIZE );
    printf ( "%d values\n", no_of_values );

    if ( block_no++ > 20 )
      {
      block_no = 0;
      printf ( "Press any key to stop\n" );
      }

    /* Print out the first 10 readings
     */
    for ( i = 0; i < no_of_values; i++ )
    {
      fprintf ( fp, "%d\n", adc_to_mv (unitOpened.values[PS3000_CHANNEL_B][i], 
		                       unitOpened.channelSettings[PS3000_CHANNEL_B].range) );
    }

    /* Wait 100ms before asking again
     */
    Sleep ( 100 );
    }
  fclose ( fp );

  ps3000_stop ( unitOpened.handle );

  getch ();
  }

void collect_fast_streaming (void)
  {
  int		i;
  int		block_no;
  FILE 	*fp;
  int		no_of_values;
  short  overflow;
  int 	ok;

  printf ( "Collect streaming...\n" );

  set_defaults ();

  /* You cannot use triggering for the start of the data...
   */
  ps3000_set_trigger ( unitOpened.handle, PS3000_NONE, 0, 0, 0, 0 );


  ok = ps3000_run_streaming_ns(unitOpened.handle,
		PS3000_US,                     //unitOpened..sample_setting.interval,
        PS3000_US, 
		100000,                       // 2,500,000 == 2.5 seconds.  
		1,                             // unitOpened.sample_setting.auto_stop,
		1,                             // (unsigned long)denominator,
		18349);

  printf ( "OK: %d\n", ok );

  no_of_values = ps3000_get_streaming_last_values(unitOpened.handle,
				(GetOverviewBuffersMaxMin)StreamingGetOverviewBuffersMaxMin);




  /* From here on, we can get data whenever we want...  */
#ifdef comment_out01
  block_no = 0;
  fp = fopen ( "data.txt", "w" );
  while ( !kbhit () )
    {
    no_of_values = ps3000_get_values ( unitOpened.handle, 
			                                 unitOpened.values[PS3000_CHANNEL_A],
											 unitOpened.values[PS3000_CHANNEL_B],
											 unitOpened.values[PS3000_CHANNEL_C],
											 unitOpened.values[PS3000_CHANNEL_D],
											 &overflow,
											 BUFFER_SIZE );
    printf ( "%d values\n", no_of_values );

    if ( block_no++ > 20 )
      {
      block_no = 0;
      printf ( "Press any key to stop\n" );
      }

    /* Print out the first 10 readings
     */
    for ( i = 0; i < no_of_values; i++ )
    {
      fprintf ( fp, "%d\n", adc_to_mv (unitOpened.values[PS3000_CHANNEL_B][i], 
		         nitOpened.channelSettings[PS3000_CHANNEL_B].range) );
    }

    /* Wait 100ms before asking again
     */
    Sleep ( 100 );
    }
  fclose ( fp );

#endif  /*comment_out01*/ 

//  ps3000_stop ( unitOpened.handle );

  getch ();
  }/* collect_fast_streaming*/  


short ps3000_callback_func (short handle, short *dataBuffer, short noOfBuffers)
{
	return 1;
}



void StreamingGetOverviewBuffersMaxMin (
                short **overviewBuffers,
                short overflow,
                unsigned long triggeredAt,
                short triggered,
                short auto_stop,
                unsigned long nValues)
				
								
{
 printf ("stub. StreamingGetOverviewBuffersMaxMin \n"); 				
								
								
} /* StreamingGetOverviewBuffersMaxMin */ 


/****************************************************************************
 *
 *
 ****************************************************************************/
void get_info (void)
  {
  char description [6][25]=  { "Driver Version","USB Version","Hardware Version",
                              "Variant Info","Serial", "Error Code" };
  short 		i;
  char		line [80];
	int    variant;

  if( unitOpened.handle )
  {
    for ( i = 0; i < 5; i++ )
    {
      ps3000_get_unit_info ( unitOpened.handle, line, sizeof (line), i );
			if (i == 3)
			{
			  variant = atoi(line);
			}
      printf ( "%s: %s\n", description[i], line );
    }

  	switch (variant)
		{
		case MODEL_PS3206:
			unitOpened.model = MODEL_PS3206;
			unitOpened.external = TRUE;
			unitOpened.signalGenerator = TRUE;
			unitOpened.firstRange = PS3000_100MV;
			unitOpened.lastRange = PS3000_20V;
			unitOpened.timebases = PS3206_MAX_TIMEBASE;
			unitOpened.noOfChannels = DUAL_SCOPE;
		break;

		case MODEL_PS3205:
			unitOpened.model = MODEL_PS3205;
			unitOpened.external = TRUE;
			unitOpened.signalGenerator = TRUE;
			unitOpened.firstRange = PS3000_100MV;
			unitOpened.lastRange = PS3000_20V;
			unitOpened.timebases = PS3205_MAX_TIMEBASE;
			unitOpened.noOfChannels = DUAL_SCOPE; 
		break;
				
		case MODEL_PS3204:
			unitOpened.model = MODEL_PS3204;
			unitOpened.external = TRUE;
			unitOpened.signalGenerator = TRUE;
	  	unitOpened.firstRange = PS3000_100MV;
			unitOpened.lastRange = PS3000_20V;
			unitOpened.timebases = PS3204_MAX_TIMEBASE;
			unitOpened.noOfChannels = DUAL_SCOPE;
		break;
				
		case MODEL_PS3224:
			unitOpened.model = MODEL_PS3224;
			unitOpened.external = FALSE;
			unitOpened.signalGenerator = FALSE;
			unitOpened.firstRange = PS3000_20MV;
			unitOpened.lastRange = PS3000_20V;
			unitOpened.timebases = PS3224_MAX_TIMEBASE;
			unitOpened.noOfChannels = DUAL_SCOPE;
		break;

		case MODEL_PS3424:
			unitOpened.model = MODEL_PS3424;
			unitOpened.external = FALSE;
			unitOpened.signalGenerator = FALSE;
			unitOpened.firstRange = PS3000_20MV;
			unitOpened.lastRange = PS3000_20V;
			unitOpened.timebases = PS3424_MAX_TIMEBASE;
			unitOpened.noOfChannels = QUAD_SCOPE;					
		break;

 		default:
			printf("Unit not supported");
		}
  }
  else
  {
    printf ( "Unit Not Opened\n" );
    ps3000_get_unit_info ( unitOpened.handle, line, sizeof (line), 5 );
    printf ( "%s: %s\n", description[5], line );
		unitOpened.model = MODEL_NONE;
		unitOpened.external = TRUE;
		unitOpened.signalGenerator = TRUE;
		unitOpened.firstRange = PS3000_10MV;
		unitOpened.lastRange = PS3000_20V;
		unitOpened.timebases = PS3206_MAX_TIMEBASE;
		unitOpened.noOfChannels = QUAD_SCOPE;	
	}
}

/****************************************************************************
 *
 * Select timebase, set oversample to on and time units as nano seconds
 *
 ****************************************************************************/
void set_timebase (void)
  {
  short	i;
  long   time_interval;
  short  time_units;
  short 	oversample;
  long   max_samples;

  printf ( "Specify timebase\n" );

  /* See what ranges are available...
   */
	oversample = 1;
  for (i = 0; i < unitOpened.timebases; i++)
    {
    ps3000_get_timebase ( unitOpened.handle, i, BUFFER_SIZE, &time_interval, 
		   &time_units, oversample, &max_samples );
    if ( time_interval > 0 )
      {
      printf ( "%d -> %d %s  %hd\n", i, time_interval, adc_units(time_units), time_units );
      }
    }

  /* Ask the user to select a timebase
   */
  printf ( "Timebase: " );
  do
  {
    fflush( stdin );
    scanf ( "%d", &timebase );
  } while ( (timebase < 0) || (timebase >= unitOpened.timebases) );
  ps3000_get_timebase ( unitOpened.handle, i, BUFFER_SIZE, &time_interval, 
	               &time_units, oversample, &max_samples );
  printf ( "Timebase %d - %ld ns\n", timebase, time_interval );
  }

/****************************************************************************
 * Select input voltage ranges for channels A and B
 ****************************************************************************/
void set_voltages (void)
{
  int		i;
  int		ch;

  /* See what ranges are available...
   */
  for ( i = unitOpened.firstRange; i <= unitOpened.lastRange; i++ )
  {
    printf ( "%d -> %d mV\n", i, input_ranges[i] );
  }

  /* Ask the user to select a range
   */
	
  printf ( "Specify voltage range (%d..%d)\n", unitOpened.firstRange, (unitOpened.lastRange) );
	printf ( "99 - switches channel off\n");
  for ( ch = 0; ch < unitOpened.noOfChannels; ch++ )
  {
    printf ( "\nChannel %c: ", 'A' + ch );
    do
    {
       fflush( stdin );
       scanf ( "%d", &unitOpened.channelSettings[ch].range);
    } while ( unitOpened.channelSettings[ch].range != 99 && 
		       (unitOpened.channelSettings[ch].range < unitOpened.firstRange || 
			   unitOpened.channelSettings[ch].range  > unitOpened.lastRange) );
		if(unitOpened.channelSettings[ch].range != 99)
		{
      printf ( " - %d mV\n", input_ranges[unitOpened.channelSettings[ch].range]);
      unitOpened.channelSettings[ch].enabled = TRUE;
		}
		else
		{
      printf ( "Channel Switched off\n", input_ranges[unitOpened.channelSettings[ch].range]);
      unitOpened.channelSettings[ch].enabled = FALSE;
    }
  }
}	


/****************************************************************************
 * Toggles the signal generator to 1kHz sine wave or off
 ***************************************************************************/
void set_signal_generator (void)
  {
  short waveform;
  char  sweep;
  long 	sig_gen_finish;
  short repeat, dwell_time, dual_slope;
  float 	increment;

	if(unitOpened.model != MODEL_PS3224 && unitOpened.model != MODEL_PS3424)
	{
	  printf("Signal generator not available for the PS%d\n", unitOpened.model);
	  return;
	}

  increment = repeat = dwell_time = dual_slope = 0;

  // Ask user to enter sig frequency;
  printf ( "Enter frequency in Hz: " );
  do
    {
    scanf( "%lu", &sig_gen_frequency );
    } while ( sig_gen_frequency < 0 || sig_gen_frequency > PS3000_MAX_SIGGEN_FREQ );

  if ( sig_gen_frequency > 0 )
    {
    printf ( "Signal generator On" );
    // Ask user to enter type of signal
    printf ( "Enter type of waveform\n" );
    printf ( "0:\tSQUARE\n" );
  	 printf ( "1:\tTRIANGLE\n" );
    printf ( "2:\tSINE\n" );
    do
      {
      scanf ( "%hd", &waveform );
      } while ( waveform < 0 || waveform >= PS3000_MAX_WAVE_TYPES );

    printf ( "Use sweep mode:" );
    fflush ( stdin );
    scanf ( "%c", &sweep );
    fflush ( stdin );
    sweep = toupper ( sweep );

    if (sweep == 'Y')
      {
      printf ( "Enter finish frequency (Hz) " );
      scanf ( "%lu", &sig_gen_finish );
      printf ( "Enter increment " );
      scanf ( "%f", &increment );
      printf ( "Enter dwell time (ns) " );
      scanf ( "%d", &dwell_time );
      printf ( "Repeat sweep (0-No, 1-Yes) " );
      scanf ( "%d", &repeat );
      printf ( "Enter dual slope (0-No, 1-Yes) " );
      scanf ( "%d", &dual_slope );
      }
	 else
      sig_gen_finish = sig_gen_frequency;
    }
  else
    {
    waveform = 0;
    printf ( "Signal generator Off" );
    }

  printf ( "Actual frequency generated is %u Hz\n", ps3000_set_siggen ( unitOpened.handle,
                                                                        waveform,
  		                                                                  sig_gen_frequency,
                                                                        sig_gen_finish,
                                                                        increment,
                                                                        dwell_time,
                                                                        repeat,
                                                                        dual_slope) );
  }


/****************************************************************************
 *
 *
 ****************************************************************************/

void main (void)
  {
  char	ch;
  int 	i;

  printf ( "PS3000 driver example program\n" );
  printf ( "Version 1.0\n\n" );

  printf ( "\n\nOpening the device...\n");

  //open unit and show splash screen
  unitOpened.handle = ps3000_open_unit ();
  printf ( "Handler: %d\n", unitOpened.handle );
  if ( !unitOpened.handle )
    {
    printf ( "Unable to open device\n" );
    get_info ();
    while( !kbhit() );
	exit ( 99 );
    }
  else
    {
    printf ( "Device opened successfully\n\n" );
    get_info ();

		timebase = 0;

		for (i = 0; i < MAX_CHANNELS; i++)
		{
			unitOpened.channelSettings[i].enabled = TRUE;
      unitOpened.channelSettings[i].DCcoupled = TRUE;
			unitOpened.channelSettings[i].range = unitOpened.lastRange;
		}

    ch = ' ';
    while ( ch != 'X' )
      {
      printf ( "\n" );
      printf ( "B - immediate block	      V - Set voltages\n" );
      printf ( "T - triggered block	      I - Set timebase\n" );
      printf ( "E - ETS block                 F - toggle signal generator on/off\n" );
      printf ( "S - Streaming                 A - ADC counts/mV\n" );
      printf ( "L - Loki 5 second grab        X - exit\n" );
      printf ( "Operation:" );

      ch = toupper ( getch () );

      printf ( "\n\n" );
      switch ( ch )
        {
        case 'B':
        collect_block_immediate ();
        break;

        case 'T':
        collect_block_triggered ();
        break;

        case 'S':
        collect_streaming ();
        break;

        case 'F':
        set_signal_generator ();
        break;

        case 'L':
        collect_fast_streaming ();
        break;

        case 'E':
        collect_block_ets ();
        break;

        case 'V':
        set_voltages ();
        break;

        case 'I':
        set_timebase ();
        break;

        case 'A':
        scale_to_mv = !scale_to_mv;
        if ( scale_to_mv )
          {
          printf ( "Readings will be scaled in mV\n" );
          }
        else
          {
          printf ( "Readings will be scaled in ADC counts\n" );
          }
        break;

        case 'X':
        /* Handled by outer loop */
        break;

        default:
        printf ( "Invalid operation\n" );
        break;
        }
      }

    ps3000_close_unit ( unitOpened.handle );
    }
  }
